def parser(subparser):

    ans = subparser.add_parser(
        "install",
        help="Install components of easifem.",
        description="""
        The [install] subcommand helps you installing the components of \n 
        EASIFEM, such as extpkgs, base, classes, materials, kernels, etc. \n
        \n
        In order to install a component you should specify following environment variables: \n
        A) EASIFEM_ROOT_DIR: the place where easifem is installed. \n
        B) EASIFEM_BUILD_DIR: the place where easifem is build. \n
        C) EASIFEM_SOURCE_DIR: the place where the source of easifem will be stored. \n
        \n
        You can specify them by using \n
        \n
        easifem setenv --build= --install= --source=
        \n
        For more see,\n
        easifem setenv --help \n
        \n
        """,
    )

    ans.add_argument(
        "-i",
        "--install",
        help="Location where easifem is installed, EASIFEM_INSTALL_DIR",
        required=False,
    )

    ans.add_argument(
        "-b",
        "--build",
        help="Location where easifem will be build, EASIFEM_BUILD_DIR",
        required=False,
    )

    ans.add_argument(
        "-s",
        "--source",
        help="Location where the source-code of easifem will be stored, EASIFEM_SOURCE_DIR",
        required=False,
    )

    ans.add_argument(
        "-q",
        "--quite",
        help="If specified lesser output will be printed",
        action="store_true",
    )

    ans.add_argument(
        "components",
        metavar="c",
        type=str,
        nargs="+",
        help="Names of components to install",
    )

    return ans



import os
import sys
import shutil
import subprocess
from easifem.console import console
from rich.panel import Panel
from rich.text import Text
from rich.progress import Progress, BarColumn, TextColumn

_easifem = "easifem"

_extpkgs = ["openblas", "lapack95", "sparsekit", "fftw", "superlu", "arpack", "lis"]

_components = ["extpkgs", "base", "classes", "materials", "kernels"] + _extpkgs

# _build0 = os.path.join(os.environ["HOME"], "temp")
# _install0 = os.environ["HOME"]
# _source0 = os.path.join(os.environ["HOME"], "code")
_build = os.getenv("EASIFEM_BUILD_DIR", "")
_install = os.getenv("EASIFEM_INSTALL_DIR", "")
_source = os.getenv("EASIFEM_SOURCE_DIR", "")

EASIFEM_EXTPKGS = os.getenv("EASIFEM_EXTPKGS", "")
EASIFEM_BASE = os.getenv("EASIFEM_BASE", "")
EASIFEM_CLASSES = os.getenv("EASIFEM_CLASSES", "")

#############################################################################
## easifem_install ##########################################################
#############################################################################


def easifem_install(
    components,
    quite=False,
    install=_install,
    build=_build,
    source=_source,
):

    if _build == "":
        console.print(Panel.fit("❌ ERROR"))
        console.print(f"Environment variable EASIFEM_BUILD_DIR not found")
        console.print("Make sure you have sourced the easifemvar.sh(fish)")
        raise SystemExit(2)

    if _install == "":
        console.print(Panel.fit("❌ ERROR"))
        console.print(f"Environment variable EASIFEM_INSTALL_DIR not found")
        console.print("Make sure you have sourced the easifemvar.sh(fish)")
        raise SystemExit(2)

    if _source == "":
        console.print(Panel.fit("❌ ERROR"))
        console.print(f"Environment variable EASIFEM_SOURCE_DIR not found")
        console.print("Make sure you have sourced the easifemvar.sh(fish)")
        raise SystemExit(2)

    if EASIFEM_EXTPKGS == "":
        console.print(Panel.fit("❌ ERROR"))
        console.print(f"Environment variable EASIFEM_EXTPKGS not found")
        console.print("Make sure you have sourced the easifemvar.sh(fish)")
        raise SystemExit(2)

    if EASIFEM_BASE == "":
        console.print(Panel.fit("❌ ERROR"))
        console.print(f"Environment variable EASIFEM_BASE not found")
        console.print("Make sure you have sourced the easifemvar.sh(fish)")
        raise SystemExit(2)

    if EASIFEM_CLASSES == "":
        console.print(Panel.fit("❌ ERROR"))
        console.print(f"Environment variable EASIFEM_CLASSES not found")
        console.print("Make sure you have sourced the easifemvar.sh(fish)")
        raise SystemExit(2)

    for c in components:
        if c in _components:
            console.print(f"\n", justify="left", style="bold green underline")
            console.print(f"\n\nInstalling {c}\n", justify="left", style="bold green")
            console.print(f"\n", justify="left", style="bold green underline")
            if c == "extpkgs":
                _install_extpkgs(
                    quite=quite,
                    install=_install if install == None else install,
                    build=_build if build == None else build,
                    source=_source if source == None else source,
                )
            if c == "openblas":
                _install_openblas(
                    quite=quite,
                    install=_install if install == None else install,
                    build=_build if build == None else build,
                    source=_source if source == None else source,
                )
            elif c == "lapack95":
                _install_lapack95(
                    quite=quite,
                    install=_install if install == None else install,
                    build=_build if build == None else build,
                    source=_source if source == None else source,
                )
            elif c == "sparsekit":
                _install_sparsekit(
                    quite=quite,
                    install=_install if install == None else install,
                    build=_build if build == None else build,
                    source=_source if source == None else source,
                )
            elif c == "fftw":
                _install_fftw(
                    quite=quite,
                    install=_install if install == None else install,
                    build=_build if build == None else build,
                    source=_source if source == None else source,
                )
            elif c == "superlu":
                _install_superlu(
                    quite=quite,
                    install=_install if install == None else install,
                    build=_build if build == None else build,
                    source=_source if source == None else source,
                )
            elif c == "arpack":
                _install_arpack(
                    quite=quite,
                    install=_install if install == None else install,
                    build=_build if build == None else build,
                    source=_source if source == None else source,
                )
            elif c == "lis":
                _install_lis(
                    quite=quite,
                    install=_install if install == None else install,
                    build=_build if build == None else build,
                    source=_source if source == None else source,
                )
            elif c == "base":
                _install_base(
                    quite=quite,
                    install=_install if install == None else install,
                    build=_build if build == None else build,
                    source=_source if source == None else source,
                )
            elif c == "classes":
                _install_classes(
                    quite=quite,
                    install=_install if install == None else install,
                    build=_build if build == None else build,
                    source=_source if source == None else source,
                )

        else:
            console.print(Panel.fit("❌ ERROR 2"))
            console.print(f"Component {c} cannot be installed\n")
            console.print(f"Currently, only {_components} can be installed\n")
            raise SystemExit(2)


#############################################################################
## runCommand ###############################################################
#############################################################################


def runCommand(cargs, quite=False):
    """
    This program runs a command,
    cargs[0] will change on return
    """
    _temp = shutil.which(cargs[0])

    if _temp:
        cargs[0] = _temp.strip()
    else:
        console.print(Panel.fit("❌ ERROR 1"))
        console.print(f"{cargs[0]} not found on system PATH")
        raise SystemExit(2)

    if not quite:
        console.print(f"=> Calling {cargs[0]} with following args ...")
        console.print("=> " + " ".join(cargs))

    p = subprocess.run(
        cargs,
        shell=False,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        timeout=1000000,
        check=False,
        encoding="utf-8",
        errors="strict",
        text=True,
        universal_newlines=True,
    )

    if p.returncode != 0:
        console.print(Panel.fit("❌ ERROR 2"))
        console.print(f"Command {cargs[0]} Failed with following message:\n")
        console.print("=> " + p.stdout)
        console.print("=> " + p.stderr)
        raise SystemExit(2)
    else:
        if not quite:
            console.print(Panel.fit("✔️R SUCCESS"))
            console.print(f"  {id} => command {cargs[0]} was successful")
            console.print(f"\n {id} => Command output: \n")
            console.print(p.stdout)


#############################################################################
## extpkgs ##################################################################
#############################################################################


def _install_extpkgs(quite, install, build, source):
    easifem_install(
        components=_extpkgs,
        quite=quite,
        install=install,
        build=build,
        source=source,
    )


#############################################################################
##  openblas ################################################################
#############################################################################


def _install_openblas(quite, install, build, source):
    print(quite, install, build, source)
    console.print("openblas is Currently not avaiable!")


#############################################################################
## lapack95 #################################################################
#############################################################################


def _install_lapack95(quite, install, build, source):
    pkg_dir = os.path.join(source, "easifem", "extpkgs")
    pkg_name = "LAPACK95"
    pkg_with_path = os.path.join(pkg_dir, pkg_name)
    url = "https://github.com/vickysharma0812/" + pkg_name + ".git"
    cmake_def = []
    cmake_def.append("-D USE_OpenMP:BOOL=ON")
    cmake_def.append("-D CMAKE_BUILD_TYPE:STRING=Release")
    cmake_def.append("-D BUILD_SHARED_LIBS:BOOL=ON")
    cmake_def.append(f"-D CMAKE_INSTALL_PREFIX:PATH={EASIFEM_EXTPKGS}")

    build_dir = os.path.join(build, "easifem", "extpkgs", pkg_name, "build")

    cwd = os.getcwd()

    if os.path.exists(pkg_with_path):
        os.chdir(pkg_with_path)
        cargs = ["git", "pull"]
        runCommand(cargs=cargs, quite=quite)
        # os.system(f"git pull")
    else:
        console.print(f"Path {pkg_with_path} does not exist.")
        console.print(f"Creating {pkg_with_path}")
        cargs = ["git", "clone", f"{url}", f"{pkg_with_path}"]
        runCommand(cargs=cargs, quite=quite)
        # os.system(f"git clone {url} {pkg_with_path}")
        os.chdir(pkg_with_path)

    os.makedirs(build_dir, exist_ok=True)

    os.makedirs(EASIFEM_EXTPKGS, exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "include"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "lib"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "bin"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "share"), exist_ok=True)
    cargs = [
        "cmake",
        "-G",
        "Ninja",
        "-S",
        f"{pkg_with_path}",
        "-B",
        f"{build_dir}",
    ] + cmake_def

    runCommand(cargs=cargs, quite=quite)
    cargs = [
        "cmake",
        "--build",
        f"{build_dir}",
        "--target",
        "install",
    ]
    runCommand(cargs=cargs, quite=quite)
    # os.system(f"cmake -S ./ -B {build_dir} {cmake_def}")
    # os.system(f"cmake --build {build_dir} --target install")

    text = Text()
    text.append(
        Text.assemble(
            ("Package Name: ", "bold yellow"),
            f"{pkg_name}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Package URL: ", "bold yellow"),
            f"{url}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Source Dir: ", "bold yellow"),
            f"{pkg_dir}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Build Dir: ", "bold yellow"),
            f"{build_dir}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Install Dir: ", "bold yellow"),
            f"{EASIFEM_EXTPKGS}\n",
        )
    )
    console.print(Panel(text, title=f"[red] {pkg_name} Info"))

    os.chdir(cwd)


#############################################################################
## sparsekit ################################################################
#############################################################################


def _install_sparsekit(quite, install, build, source):
    pkg_dir = os.path.join(source, "easifem", "extpkgs")
    pkg_name = "Sparsekit"
    pkg_with_path = os.path.join(pkg_dir, pkg_name)
    url = "https://github.com/vickysharma0812/" + pkg_name + ".git"

    cmake_def = []
    cmake_def.append("-D CMAKE_BUILD_TYPE:STRING=Release ")
    cmake_def.append("-D BUILD_SHARED_LIBS:BOOL=ON ")
    cmake_def.append(f"-D CMAKE_INSTALL_PREFIX:PATH={EASIFEM_EXTPKGS}")

    build_dir = os.path.join(build, "easifem", "extpkgs", pkg_name, "build")

    cwd = os.getcwd()

    if os.path.exists(pkg_with_path):
        os.chdir(pkg_with_path)
        cargs = ["git", "pull"]
        runCommand(cargs=cargs, quite=quite)
        # os.system(f"git pull")
    else:
        console.print(f"Path {pkg_with_path} does not exist.")
        console.print(f"Creating {pkg_with_path}")
        cargs = ["git", "clone", f"{url}", f"{pkg_with_path}"]
        runCommand(cargs=cargs, quite=quite)
        # os.system(f"git clone {url} {pkg_with_path}")
        os.chdir(pkg_with_path)

    os.makedirs(build_dir, exist_ok=True)
    os.makedirs(EASIFEM_EXTPKGS, exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "include"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "lib"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "bin"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "share"), exist_ok=True)
    cargs = [
        "cmake",
        "-G",
        "Ninja",
        "-S",
        f"{pkg_with_path}",
        "-B",
        f"{build_dir}",
    ] + cmake_def

    runCommand(cargs=cargs, quite=quite)
    cargs = [
        "cmake",
        "--build",
        f"{build_dir}",
        "--target",
        "install",
    ]
    runCommand(cargs=cargs, quite=quite)
    # os.system(f"cmake -S ./ -B {build_dir} {cmake_def}")
    # os.system(f"cmake --build {build_dir} --target install")

    text = Text()
    text.append(
        Text.assemble(
            ("Package Name: ", "bold yellow"),
            f"{pkg_name}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Package URL: ", "bold yellow"),
            f"{url}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Source Dir: ", "bold yellow"),
            f"{pkg_dir}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Build Dir: ", "bold yellow"),
            f"{build_dir}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Install Dir: ", "bold yellow"),
            f"{EASIFEM_EXTPKGS}\n",
        )
    )
    console.print(Panel(text, title=f"[red] {pkg_name} Info"))

    os.chdir(cwd)


#############################################################################
## fftw #####################################################################
#############################################################################


def _install_fftw(quite, install, build, source):
    pkg_dir = os.path.join(source, "easifem", "extpkgs")
    pkg_name = "fftw-3.3.10"
    pkg_with_path = os.path.join(pkg_dir, pkg_name)
    url = "https://www.fftw.org/fftw-3.3.10.tar.gz"
    compress_file = os.path.join(pkg_dir, "fftw.tar.gz")

    cwd = os.getcwd()

    if os.path.exists(pkg_with_path):
        os.chdir(pkg_with_path)
    else:
        if not os.path.isfile(compress_file):
            cargs = [
                "curl",
                "-L",
                url,
                "-o",
                compress_file,
            ]

        runCommand(cargs=cargs, quite=quite)
        cargs = [
            "tar",
            "-xf",
            compress_file,
            "-C",
            pkg_dir,
        ]
        runCommand(cargs=cargs, quite=quite)
        os.chdir(pkg_with_path)

    os.makedirs(EASIFEM_EXTPKGS, exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "include"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "lib"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "bin"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "share"), exist_ok=True)
    cargs = [
        os.path.join(pkg_with_path, "configure"),
        "--prefix",
        EASIFEM_EXTPKGS,
        "--enable-openmp",
    ]
    runCommand(cargs=cargs, quite=quite)
    cargs = [
        "make",
    ]
    runCommand(cargs=cargs, quite=quite)

    cargs = [
        "make",
        "install",
    ]
    runCommand(cargs=cargs, quite=quite)

    text = Text()
    text.append(
        Text.assemble(
            ("Package Name: ", "bold yellow"),
            f"{pkg_name}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Package URL: ", "bold yellow"),
            f"{url}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Source Dir: ", "bold yellow"),
            f"{pkg_dir}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Install Dir: ", "bold yellow"),
            f"{EASIFEM_EXTPKGS}\n",
        )
    )
    console.print(Panel(text, title=f"[red] {pkg_name} Info"))

    os.chdir(cwd)


#############################################################################
## superlu ##################################################################
#############################################################################


def _install_superlu(quite, install, build, source):
    pkg_dir = os.path.join(source, "easifem", "extpkgs")
    pkg_name = "superlu"
    pkg_with_path = os.path.join(pkg_dir, pkg_name)
    url = "https://github.com/xiaoyeli/" + pkg_name + ".git"
    cmake_def = []
    cmake_def.append("-D CMAKE_BUILD_TYPE:STRING=Release")
    cmake_def.append("-D BUILD_SHARED_LIBS:BOOL=ON")
    cmake_def.append("-D TPL_ENABLE_METISLIB:BOOL=OFF")
    cmake_def.append(f"-D CMAKE_INSTALL_PREFIX:PATH={EASIFEM_EXTPKGS}")

    build_dir = os.path.join(build, "easifem", "extpkgs", pkg_name, "build")

    cwd = os.getcwd()

    if os.path.exists(pkg_with_path):
        os.chdir(pkg_with_path)
        cargs = ["git", "pull"]
        runCommand(cargs=cargs, quite=quite)
        # os.system(f"git pull")
    else:
        console.print(f"Path {pkg_with_path} does not exist.")
        console.print(f"Creating {pkg_with_path}")
        console.print(f"cloning {pkg_name} from git.")
        cargs = ["git", "clone", f"{url}", f"{pkg_with_path}"]
        runCommand(cargs=cargs, quite=quite)
        os.chdir(pkg_with_path)

    os.makedirs(build_dir, exist_ok=True)
    os.makedirs(EASIFEM_EXTPKGS, exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "include"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "lib"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "bin"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "share"), exist_ok=True)
    cargs = [
        "cmake",
        "-G",
        "Ninja",
        "-S",
        f"{pkg_with_path}",
        "-B",
        f"{build_dir}",
    ] + cmake_def

    runCommand(cargs=cargs, quite=quite)
    cargs = [
        "cmake",
        "--build",
        f"{build_dir}",
        "--target",
        "install",
    ]
    runCommand(cargs=cargs, quite=quite)
    # os.system(f"cmake -S ./ -B {build_dir} {cmake_def}")
    # os.system(f"cmake --build {build_dir} --target install")

    text = Text()
    text.append(
        Text.assemble(
            ("Package Name: ", "bold yellow"),
            f"{pkg_name}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Package URL: ", "bold yellow"),
            f"{url}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Source Dir: ", "bold yellow"),
            f"{pkg_dir}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Build Dir: ", "bold yellow"),
            f"{build_dir}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Install Dir: ", "bold yellow"),
            f"{EASIFEM_EXTPKGS}\n",
        )
    )
    console.print(Panel(text, title=f"[red] {pkg_name} Info"))

    os.chdir(cwd)


#############################################################################
## arpac ####################################################################
#############################################################################


def _install_arpack(quite, install, build, source):
    pkg_dir = os.path.join(source, "easifem", "extpkgs")
    pkg_name = "arpack-ng"
    pkg_with_path = os.path.join(pkg_dir, pkg_name)
    url = "https://github.com/opencollab/" + pkg_name + ".git"
    cmake_def = []
    cmake_def.append("-D MPI:BOOL=OFF")
    cmake_def.append("-D CMAKE_BUILD_TYPE:STRING=Release")
    cmake_def.append("-D BUILD_SHARED_LIBS:BOOL=ON")
    cmake_def.append(f"-D CMAKE_INSTALL_PREFIX:PATH={EASIFEM_EXTPKGS}")

    build_dir = os.path.join(build, "easifem", "extpkgs", pkg_name, "build")

    cwd = os.getcwd()

    if os.path.exists(pkg_with_path):
        os.chdir(pkg_with_path)
        cargs = ["git", "pull"]
        runCommand(cargs=cargs, quite=quite)
        # os.system(f"git pull")
    else:
        console.print(f"Path {pkg_with_path} does not exist.")
        console.print(f"Creating {pkg_with_path}")
        cargs = ["git", "clone", f"{url}", f"{pkg_with_path}"]
        runCommand(cargs=cargs, quite=quite)
        # os.system(f"git clone {url} {pkg_with_path}")
        os.chdir(pkg_with_path)

    os.makedirs(build_dir, exist_ok=True)
    os.makedirs(EASIFEM_EXTPKGS, exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "include"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "lib"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "bin"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "share"), exist_ok=True)
    cargs = [
        "cmake",
        "-G",
        "Ninja",
        "-S",
        f"{pkg_with_path}",
        "-B",
        f"{build_dir}",
    ] + cmake_def

    runCommand(cargs=cargs, quite=quite)
    cargs = [
        "cmake",
        "--build",
        f"{build_dir}",
        "--target",
        "install",
    ]
    runCommand(cargs=cargs, quite=quite)
    # os.system(f"cmake -S ./ -B {build_dir} {cmake_def}")
    # os.system(f"cmake --build {build_dir} --target install")

    text = Text()
    text.append(
        Text.assemble(
            ("Package Name: ", "bold yellow"),
            f"{pkg_name}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Package URL: ", "bold yellow"),
            f"{url}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Source Dir: ", "bold yellow"),
            f"{pkg_dir}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Build Dir: ", "bold yellow"),
            f"{build_dir}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Install Dir: ", "bold yellow"),
            f"{EASIFEM_EXTPKGS}\n",
        )
    )
    console.print(Panel(text, title=f"[red] {pkg_name} Info"))

    os.chdir(cwd)


#############################################################################
## lis ######################################################################
#############################################################################


def _install_lis(quite, install, build, source):
    pkg_dir = os.path.join(source, "easifem", "extpkgs")
    pkg_name = "lis"
    pkg_with_path = os.path.join(pkg_dir, pkg_name)
    url = "https://github.com/anishida/" + pkg_name + ".git"
    cmake_def = []
    cmake_def.append(f"--prefix={EASIFEM_EXTPKGS}")
    cmake_def.append("--enable-omp")
    cmake_def.append("--enable-f90")
    cmake_def.append("--enable-shared")
    cmake_def.append("--enable-saamg")
    cmake_def.append("FC=gfortran")
    cmake_def.append("CC=gcc")

    cwd = os.getcwd()

    if os.path.exists(pkg_with_path):
        os.chdir(pkg_with_path)
        cargs = ["git", "pull"]
        runCommand(cargs=cargs, quite=quite)
        # os.system(f"git pull")
    else:
        console.print(f"Path {pkg_with_path} does not exist.")
        console.print(f"Creating {pkg_with_path}")
        cargs = ["git", "clone", f"{url}", f"{pkg_with_path}"]
        runCommand(cargs=cargs, quite=quite)
        # os.system(f"git clone {url} {pkg_with_path}")
        os.chdir(pkg_with_path)

    os.makedirs(EASIFEM_EXTPKGS, exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "include"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "lib"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "bin"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_EXTPKGS, "share"), exist_ok=True)

    cargs = [
        os.path.join(pkg_with_path, "configure"),
    ] + cmake_def
    runCommand(cargs=cargs, quite=quite)

    cargs = ["make"]
    runCommand(cargs=cargs, quite=quite)

    cargs = ["make", "check"]
    runCommand(cargs=cargs, quite=quite)

    cargs = ["make", "install"]
    runCommand(cargs=cargs, quite=quite)

    cargs = ["make", "installcheck"]
    runCommand(cargs=cargs, quite=quite)

    text = Text()
    text.append(
        Text.assemble(
            ("Package Name: ", "bold yellow"),
            f"{pkg_name}\n",
        )
    )

    text.append(
        Text.assemble(
            ("Package URL: ", "bold yellow"),
            f"{url}\n",
        )
    )

    text.append(
        Text.assemble(
            ("Source Dir: ", "bold yellow"),
            f"{pkg_dir}\n",
        )
    )

    text.append(
        Text.assemble(
            ("Install Dir: ", "bold yellow"),
            f"{EASIFEM_EXTPKGS}\n",
        )
    )
    console.print(Panel(text, title=f"[red] {pkg_name} Info"))

    os.chdir(cwd)


#############################################################################
## blas #####################################################################
#############################################################################


def _install_base(quite, install, build, source):
    pkg_dir = os.path.join(source, "easifem")
    pkg_name = "base"
    pkg_with_path = os.path.join(pkg_dir, pkg_name)
    url = "https://github.com/vickysharma0812/easifem-base.git"
    cmake_def = []
    cmake_def.append("-D USE_OpenMP:BOOL=ON")
    cmake_def.append(f"-D CMAKE_INSTALL_PREFIX:PATH={EASIFEM_BASE}")
    cmake_def.append("-D CMAKE_BUILD_TYPE:STRING=Release")
    cmake_def.append("-D BUILD_SHARED_LIBS:BOOL=ON")
    cmake_def.append("-D USE_PLPLOT:BOOL=ON")
    cmake_def.append("-D USE_BLAS95:BOOL=ON")
    cmake_def.append("-D USE_LAPACK95:BOOL=ON")
    cmake_def.append("-D USE_FFTW:BOOL=ON")
    cmake_def.append("-D USE_GTK:BOOL=OFF")
    cmake_def.append("-D USE_ARPACK:BOOL=ON")
    cmake_def.append("-D USE_PARPACK:BOOL=OFF")
    cmake_def.append("-D USE_METIS:BOOL=OFF")
    cmake_def.append("-D USE_SUPERLU:BOOL=ON")
    cmake_def.append("-D USE_LIS:BOOL=ON")
    cmake_def.append("-D USE_Int32:BOOL=ON")
    cmake_def.append("-D USE_Real64:BOOL=ON")

    build_dir = os.path.join(build, "easifem", pkg_name, "build")

    cwd = os.getcwd()

    if os.path.exists(pkg_with_path):
        os.chdir(pkg_with_path)
        cargs = ["git", "pull"]
        runCommand(cargs=cargs, quite=quite)
    else:
        console.print(f"Path {pkg_with_path} does not exist.")
        console.print(f"Creating {pkg_with_path}")
        # cargs = ["git", "clone", f"{url}", f"{pkg_with_path}"]
        cargs = ["gh", "repo", "clone", "vickysharma0812/easifem-base", pkg_with_path]
        runCommand(cargs=cargs, quite=quite)
        os.chdir(pkg_with_path)

    os.makedirs(build_dir, exist_ok=True)
    os.makedirs(EASIFEM_BASE, exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_BASE, "include"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_BASE, "lib"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_BASE, "bin"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_BASE, "share"), exist_ok=True)
    cargs = [
        "cmake",
        "-G",
        "Ninja",
        "-S",
        f"{pkg_with_path}",
        "-B",
        f"{build_dir}",
    ] + cmake_def

    runCommand(cargs=cargs, quite=quite)
    cargs = [
        "cmake",
        "--build",
        f"{build_dir}",
        "--target",
        "install",
    ]
    runCommand(cargs=cargs, quite=quite)

    text = Text()
    text.append(
        Text.assemble(
            ("Package Name: ", "bold yellow"),
            f"{pkg_name}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Package URL: ", "bold yellow"),
            f"{url}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Source Dir: ", "bold yellow"),
            f"{pkg_dir}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Build Dir: ", "bold yellow"),
            f"{build_dir}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Install Dir: ", "bold yellow"),
            f"{EASIFEM_BASE}\n",
        )
    )
    console.print(Panel(text, title=f"[red] {pkg_name} Info"))

    os.chdir(cwd)


#############################################################################
## classes ##################################################################
#############################################################################


def _install_classes(quite, install, build, source):
    pkg_dir = os.path.join(source, "easifem")
    pkg_name = "classes"
    pkg_with_path = os.path.join(pkg_dir, pkg_name)
    url = "https://github.com/vickysharma0812/easifem-classes.git"

    cmake_def = []
    cmake_def.append("-D USE_OpenMP:BOOL=ON")
    cmake_def.append(f"-D CMAKE_INSTALL_PREFIX:PATH={EASIFEM_CLASSES}")
    cmake_def.append("-D CMAKE_BUILD_TYPE:STRING=Release")
    cmake_def.append("-D BUILD_SHARED_LIBS:BOOL=ON")
    cmake_def.append("-D USE_PLPLOT:BOOL=ON")
    cmake_def.append("-D USE_BLAS95:BOOL=ON")
    cmake_def.append("-D USE_LAPACK95:BOOL=ON")
    cmake_def.append("-D USE_FFTW:BOOL=ON")
    cmake_def.append("-D USE_ARPACK:BOOL=ON")
    cmake_def.append("-D USE_SUPERLU:BOOL=ON")
    cmake_def.append("-D USE_LIS:BOOL=ON")
    cmake_def.append("-D USE_Int32:BOOL=ON")
    cmake_def.append("-D USE_Real64:BOOL=ON")
    cmake_def.append("-D USE_PARPACK:BOOL=OFF")
    cmake_def.append("-D USE_METIS:BOOL=OFF")
    cmake_def.append("-D USE_GMSH_SDK:BOOL=OFF")
    cmake_def.append("-D USE_GTK:BOOL=OFF")

    build_dir = os.path.join(build, "easifem", pkg_name, "build")

    cwd = os.getcwd()

    if os.path.exists(pkg_with_path):
        os.chdir(pkg_with_path)
        cargs = ["git", "pull"]
        runCommand(cargs=cargs, quite=quite)
    else:
        console.print(f"Path {pkg_with_path} does not exist.")
        console.print(f"Creating {pkg_with_path}")
        # cargs = ["git", "clone", f"{url}", f"{pkg_with_path}"]
        cargs = [
            "gh",
            "repo",
            "clone",
            "vickysharma0812/easifem-classes",
            pkg_with_path,
        ]
        runCommand(cargs=cargs, quite=quite)
        os.chdir(pkg_with_path)

    os.makedirs(build_dir, exist_ok=True)
    os.makedirs(EASIFEM_CLASSES, exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_CLASSES, "include"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_CLASSES, "lib"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_CLASSES, "bin"), exist_ok=True)
    os.makedirs(os.path.join(EASIFEM_CLASSES, "share"), exist_ok=True)
    cargs = [
        "cmake",
        "-G",
        "Ninja",
        "-S",
        f"{pkg_with_path}",
        "-B",
        f"{build_dir}",
    ] + cmake_def

    runCommand(cargs=cargs, quite=quite)
    cargs = [
        "cmake",
        "--build",
        f"{build_dir}",
        "--target",
        "install",
    ]
    runCommand(cargs=cargs, quite=quite)

    text = Text()
    text.append(
        Text.assemble(
            ("Package Name: ", "bold yellow"),
            f"{pkg_name}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Package URL: ", "bold yellow"),
            f"{url}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Source Dir: ", "bold yellow"),
            f"{pkg_dir}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Build Dir: ", "bold yellow"),
            f"{build_dir}\n",
        )
    )
    text.append(
        Text.assemble(
            ("Install Dir: ", "bold yellow"),
            f"{EASIFEM_EXTPKGS}\n",
        )
    )
    console.print(Panel(text, title=f"[red] {pkg_name} Info"))

    os.chdir(cwd)



def parser(subparser):

    ans = subparser.add_parser(
        "setenv",
        help="Set environment variables for running easifem on your system.",
        description="""
        The [setenv] subcommand sets the environment variable \n
        for running easifem on your system. \n
        \n 
        While setting the environment you can provide following details. \n
        A) EASIFEM_INSTALL_DIR \n
        B) EASIFEM_BUILD_DIR \n
        C) EASIFEM_SOURCE_DIR \n
        \n
        EASIFEM_INSTALL_DIR: denotes the root location where EASIFEM will \n
        be installed. It is specified by \n
        --install=value. \n
        \n
        Following are the good choices for --install variable: \n
        1) ${HOME} \n
        2) ${HOME}/.local \n
        3) /opt \n
        \n
        The default value is ${HOME}.
        \n
        EASIFEM_SOURCE_DIR: specifies the location where the source code of \n
        the components of EASIFEM will be stored. \n
        It is specified by\n
        --source=value \n
        \n
        Following are the good choices for --source variable: \n
        1) ${HOME}/code/\n
        \n
        The default value is ${HOME}/code.
        \n
        EASIFEM_BUILD_DIR: specifies the location where the components of \n
        EASIFEM will be build. It is specified by\n
        --build=value \n
        \n
        Following are the good choices for --root variable: \n
        1) ${HOME}/temp \n
        \n
        The default value is ${HOME}/temp.
        \n

        Example

        easifem setenv --install ${HOME} --build ${HOME}/temp --source ${HOME}/code
        easifem setenv -r ${HOME} -b ${HOME}/temp -s ${HOME}/code
        """,
    )

    ans.add_argument(
        "-i",
        "--install",
        help="Root directory for easifem, EASIFEM_INSTALL_DIR",
        required=False,
    )

    ans.add_argument(
        "-b",
        "--build",
        help="Location where easifem will be build, EASIFEM_BUILD_DIR",
        required=False,
    )

    ans.add_argument(
        "-s",
        "--source",
        help="Location where the source-code of easifem will be stored, EASIFEM_SOURCE_DIR",
        required=False,
    )

    ans.add_argument(
        "--shell",
        help="System shell that you are using, you have following choices: [bash, zsh, fish]",
        required=False,
        choices=[
            "bash",
            "zsh",
            "fish",
        ],
    )

    ans.add_argument(
        "-q",
        "--quite",
        help="If specified lesser output will be printed",
        action="store_true",
    )

    return ans



import easifem.run
import easifem.test
import easifem.setenv
import easifem.install

testParser = easifem.test.parser
runParser = easifem.run.parser
setenvParser = easifem.setenv.parser
installParser = easifem.install.parser



import os
from easifem.console import console
from rich.panel import Panel
from rich.text import Text

_easifem = "easifem"
_keys = [
    "EASIFEM_BASE",
    "EASIFEM_CLASSES",
    "EASIFEM_EXTPKGS",
    "EASIFEM_APP",
    "EASIFEM_MATERIALS",
    "EASIFEM_KERNELS",
]

_values = [
    "base",
    "classes",
    "extpkgs",
    "app",
    "materials",
    "kernels",
]

_build0 = os.path.join(os.environ["HOME"], "temp")
_install0 = os.environ["HOME"]
_source0 = os.path.join(os.environ["HOME"], "code")


def easifem_setenv(
    quite=False,
    install=_install0,
    build=_build0,
    source=_source0,
    shell="bash",
):

    os.environ["EASIFEM_INSTALL_DIR"] = str(install)
    os.environ["EASIFEM_BUILD_DIR"] = str(build)
    os.environ["EASIFEM_SOURCE_DIR"] = str(source)

    for i in range(len(_keys)):
        os.environ[_keys[i]] = os.path.join(
            install,
            _easifem,
            _values[i],
        )

    if not quite:
        console.print(f"◌ Environment variables for EASIFEM")
        console.print(f"◌ EASIFEM_INSTALL_DIR: {os.environ.get('EASIFEM_INSTALL_DIR')}")
        console.print(f"◌ EASIFEM_BUILD_DIR: {os.environ.get('EASIFEM_BUILD_DIR')}")
        console.print(f"◌ EASIFEM_SOURCE_DIR: {os.environ.get('EASIFEM_SOURCE_DIR')}")

        for key in _keys:
            console.print(f"◌ {key}: {os.environ.get(key)}")

    _config_path = os.path.join(os.environ["HOME"], ".config", "easifem")
    os.makedirs(_config_path, exist_ok=True)

    _config_file_sh = os.path.join(_config_path, "easifemvar.sh")
    easifemrc = open(_config_file_sh, "w")
    # easifemrc.write("#!/bin/bash \n \n")
    easifemrc.write("\n")
    a = os.environ.get("EASIFEM_INSTALL_DIR")
    easifemrc.write(f"export EASIFEM_INSTALL_DIR={a} \n")
    a = os.environ.get("EASIFEM_BUILD_DIR")
    easifemrc.write(f"export EASIFEM_BUILD_DIR={a} \n")
    a = os.environ.get("EASIFEM_SOURCE_DIR")
    easifemrc.write(f"export EASIFEM_SOURCE_DIR={a} \n")
    for key in _keys:
        a = os.environ.get(key)
        easifemrc.write(f"export {key}={a} \n")
        easifemrc.write(
            "export LD_LIBRARY_PATH=" + '"${LD_LIBRARY_PATH}:${' + f"{key}" + '}/lib"\n'
        )

    easifemrc.write(
        "export PKG_CONFIG_PATH="
        + '"${PKG_CONFIG_PATH}:${EASIFEM_EXTPKGS}/lib/pkgconfig"\n'
    )
    # export PKG_CONFIG_PATH="${PKG_CONFIG_PATH}:${EASIFEM_EXTPKGS}/lib/pkgconfig"
    # export LD_LIBRARY_PATH="${LD_LIBRARY_PATH}:${EASIFEM_EXTPKGS}/lib"
    # export LD_LIBRARY_PATH="${LD_LIBRARY_PATH}:${EASIFEM_CLASSES}/lib"
    # export LD_LIBRARY_PATH="${LD_LIBRARY_PATH}:${EASIFEM_MATERIALS}/lib"
    # export LD_LIBRARY_PATH="${LD_LIBRARY_PATH}:${EASIFEM_KERNELS}/lib"
    # export LD_LIBRARY_PATH="${LD_LIBRARY_PATH}:${HOME}/.local/lib/"
    # export LD_LIBRARY_PATH="${LD_LIBRARY_PATH}:${EASIFEM_BASE}/lib"
    easifemrc.close()

    _config_file_fish = os.path.join(_config_path, "easifemvar.fish")
    easifemrc = open(_config_file_fish, "w")
    easifemrc.write("\n")
    a = os.environ.get("EASIFEM_INSTALL_DIR")
    easifemrc.write(f"set -gx EASIFEM_INSTALL_DIR {a} \n")
    a = os.environ.get("EASIFEM_BUILD_DIR")
    easifemrc.write(f"set -gx EASIFEM_BUILD_DIR {a} \n")
    a = os.environ.get("EASIFEM_SOURCE_DIR")
    easifemrc.write(f"set -gx EASIFEM_SOURCE_DIR {a} \n")
    for key in _keys:
        a = os.environ.get(key)
        easifemrc.write(f"set -gx {key} {a} \n")
    easifemrc.close()

    console.print("\n Environment variables have been written in  ↓")
    console.print(Panel.fit(f"{_config_file_sh}\n\n" + f"{_config_file_fish}"))

    text = Text()
    text.append("\n🚀 Please perform the following task: ⤵\n\n", style="bold")
    text.append(
        "⊙ Check SHELL by using following command in terminal:\n\n",
        style="italic cyan",
    )
    text.append(
        "which $SHELL \n\n",
        style="bold magenta",
    )

    text.append(
        "◌ BASH: If SHELL is bash, then add following lines to ~/.bashrc\n\n",
        style="",
    )

    text.append(
        f"source {_config_file_sh}\n\n",
        style="underline",
    )

    text.append(
        "◌ ZSH: If SHELL is zsh, then add following lines to ~/.zshrc\n\n",
        style="",
    )

    text.append(
        f"source {_config_file_sh}\n\n",
        style="underline",
    )

    console.print(
        Panel(
            text,
            title="[bold red] Modify SHELL",
        )
    )
    # console.print(f"\n🚀 Please perform the following task: ⤵")
    # console.print(f"\n⊙ Check SHELL by using following command in terminal:\n")
    # console.rule("[bold red] which $SHELL")
    # console.print("\n🔨 If SHELL is bash, then add following lines to ~/.bashrc")
    # console.print(f"source {_config_file}")
    # console.print(f"\n🔨 If SHELL is zsh, then add following lines to ~/.zshrc")
    # console.print(f"source {_config_file}")
    # if shell in ["bash", "zsh"]:



from pathlib import Path
import shutil
import subprocess
import os
from easifem.cmake import cmakeListBase
from easifem.console import console, print
from rich.panel import Panel
from rich.table import Column
from rich.progress import Progress, BarColumn, TextColumn


def _actOnFile(input: str, check=True):
    """
    ⁃ Absolute path parent : /home/easifem/Dropbox/easifem/easifem-docs/apps/python/test/
    ⁃ Absolute path        : /home/easifem/Dropbox/easifem/easifem-docs/apps/python/test/hello
    ⁃ File basename        : hello.F90
    ⁃ File ext             : .F90
    """

    path = Path(input)

    if check:
        if not path.is_file():
            print(
                """
                --file is not a file, it must be a file, 
                Also, make sure it exists
                """
            )
            raise SystemExit(2)

    # make path absolute

    absPath = path.absolute()

    return (
        str(absPath.parent),
        str(absPath.parent.joinpath(absPath.stem)),
        absPath.name,
        absPath.suffix,
    )


def runCommand(cargs, quite=False, id: str = "", lastOutput=False):
    """
    This program runs a command,
    cargs[0] will change on return
    """
    _temp = shutil.which(cargs[0])

    if _temp:
        cargs[0] = _temp.strip()
    else:
        console.print(Panel.fit(" ERROR 1"))
        console.print(f" {id} {cargs[0]} not found on system PATH")
        raise SystemExit(2)

    if not quite:
        console.print(f"=> Calling {cargs[0]} with following args ...")
        console.print("=> " + " ".join(cargs))

    p = subprocess.run(
        cargs,
        shell=False,
        stdout=subprocess.PIPE,
        stderr=subprocess.PIPE,
        timeout=100,
        check=False,
        encoding="utf-8",
        errors="strict",
        text=True,
        universal_newlines=True,
    )

    if p.returncode != 0:
        console.print(Panel.fit(" ERROR 2"))
        console.print(f" {id} command {cargs[0]} Failed with following message:\n")
        console.print("=> " + p.stdout)
        console.print("=> " + p.stderr)
        raise SystemExit(2)
    else:
        if not quite:
            console.print(Panel.fit("✔️R SUCCESS"))
            console.print(f"  {id} => command {cargs[0]} was successful")
            console.print(f"\n {id} => Command output: \n")
            console.print(p.stdout)
        else:
            if lastOutput:
                console.print(Panel.fit("✔️R SUCCESS"))
                console.print(f"\n {id} => Command output: \n")
                console.print(p.stdout)


def _makeEasifemLibsTree(easifemLibs):
    """
    Dependencies among easifem components
    """
    if easifemLibs == "easifemBase":
        return ("easifemBase",)
    elif easifemLibs == "easifemClasses":
        return (
            "easifemBase",
            "easifemClasses",
        )
    elif easifemLibs == "easifemMaterials":
        return (
            "easifemBase",
            "easifemClasses",
            "easifemMaterials",
        )
    elif easifemLibs == "easifemKernels":
        return (
            "easifemBase",
            "easifemClasses",
            "easifemMaterials",
            "easifemKernels",
        )
    elif easifemLibs == "StokesFlow":
        return (
            "easifemBase",
            "easifemClasses",
            "easifemMaterials",
            "easifemKernels",
            "StokesFlow",
        )
    elif easifemLibs == "MovingMesh":
        return (
            "easifemBase",
            "easifemClasses",
            "easifemMaterials",
            "easifemKernels",
            "MovingMesh",
        )
    else:
        raise ValueError(
            f"Given easifemLibs {easifemLibs} not supported, yet! Please use easifemBase, easifemClasses"
        )


def _makeExtpkgs(easifemLibs):
    """
    Dependencies of easifem components on external libs
    """
    return (
        "LAPACK95",
        "Sparsekit",
        "arpackng",
        "OpenMP",
    )


def easifem_run(
    file: str,
    quite=False,
    targetName="test",
    projectName="easifemApp",
    easifemLibs="easifemBase",
    buildPath=os.environ["HOME"] + "/temp/tests/build/",
    installPrefix=".",
):
    inputAbsParent, inputAbsPath, inputFileBase, inputFileExt = _actOnFile(file)

    if not quite:
        console.print(f"\n=> Following properties of input file {file} is used: ")
        console.print(f"=> Absolute parent: {inputAbsParent}")
        console.print(f"=> Absolute path  : {inputAbsPath}")
        console.print(f"=> File basename  : {inputFileBase}")
        console.print(f"=> File ext       : {inputFileExt}\n")

    progress = Progress(
        TextColumn("{task.description}", table_column=Column(ratio=1)),
        BarColumn(bar_width=None, table_column=Column(ratio=2)),
        expand=False,
        console=console,
        refresh_per_second=5,
        transient=False,
    )

    with progress:
        if inputFileExt == ".md":

            console.print(Panel.fit(" Markdown File Detected"))

            _actOnMdFile(
                inputAbsPath,
                projectName,
                targetName,
                quite,
                _makeEasifemLibsTree(easifemLibs),
                buildPath,
                installPrefix,
                progress,
            )

            # easifem test is used for testing development only
            # I keep tests generated from the markdown parser in
            # $EASIFEM_TEST_DIR
            # for testing I use installPrefix = "*--testing--*" as wild flag
            # all other values of installPrefix options will build in
            # the parent director of markdown or source file

        elif inputFileExt in [".F90", ".f90", ".f95", ".f"]:

            console.print(Panel.fit(" Fortran File Detected"))

            _actOnFortranFile(
                inputAbsPath,
                projectName,
                targetName,
                quite,
                _makeEasifemLibsTree(easifemLibs),
                buildPath,
                installPrefix,
                progress,
            )

        else:
            console.print(
                f"""
                unrecognized file extension in {inputFileExt}, should be md, F90, f90"
                """
            )
            raise SystemExit(1)


def _actOnF90File(
    outputAbsParent,
    outputAbsPath,
    outputFileBase,
    installPrefix,
    buildPath,
    targetName,
    projectName,
    easifemLibs,
    quite,
    progress,
    job,
):
    _cmakeFile = os.path.join(outputAbsParent, "CMakeLists.txt")
    cmakeFile = open(_cmakeFile, "w")
    srcPath = outputAbsParent

    for line in cmakeListBase(
        projectName=projectName,
        srcPath=srcPath,
        targetName=targetName,
        easifemLibs=easifemLibs,
        extLibs=(
            "LAPACK95",
            "Sparsekit",
            "arpackng",
            "OpenMP",
        ),
    ):
        cmakeFile.write(line + "\n")

    cmakeFile.close()

    # Go to TESTDir and Depending upon TESTDir
    # If installPrefix is *--testing--* then
    # we go to EASIFEM_TEST_DIR/parent/ and run the test
    #
    if installPrefix == "*--testing--*":
        testDir = os.environ["EASIFEM_TEST_DIR"]
        if not testDir:
            console.print(f" {id} Error env variable EASIFEM_TEST_DIR not found:\n")
            raise SystemExit(2)
        else:
            #
            # Update outputAbsParent outputAbsPath
            #
            _outputAbsParent = outputAbsParent + ""
            outputAbsParent = os.path.join(testDir, Path(outputAbsParent).stem)
            os.makedirs(outputAbsParent, exist_ok=True)
            os.rename(_cmakeFile, os.path.join(outputAbsParent, "CMakeLists.txt"))

            outputAbsPath = os.path.join(outputAbsParent, Path(outputFileBase).stem)
            os.rename(
                os.path.join(_outputAbsParent, outputFileBase),
                os.path.join(outputAbsParent, outputFileBase),
            )
    cwd = os.getcwd()
    os.chdir(outputAbsParent)

    if os.path.exists(buildPath):
        if not quite:
            console.print(f"  Found {buildPath}, removing it")
        shutil.rmtree(buildPath)

    cargs = [
        "cmake",
        "-G",
        "Ninja",
        "-B",
        buildPath,
        f"-DFILE={outputAbsPath}.F90",
    ]

    runCommand(cargs, quite=quite, id="Config App")
    # progress.advance(job)
    #
    # Command
    #
    cargs = [
        "cmake",
        "--build",
        buildPath,
    ]

    runCommand(cargs, quite=quite, id="Building App")

    progress.advance(job)
    #
    # Command
    #
    cargs = [os.path.join(buildPath, targetName)]
    runCommand(cargs, quite=True, id="Running App", lastOutput=True)

    progress.advance(job)
    #
    # console.save_svg(
    #     path=os.path.join(outputAbsParent, targetName) + ".svg",
    #     title=targetName,
    # )
    console.save_text(
        path=os.path.join(outputAbsParent, targetName) + ".txt",
        styles=False,
    )

    os.chdir(cwd)

    progress.advance(job)


def _actOnMdFile(
    inputAbsPath: str,
    projectName: str,
    targetName: str,
    quite: bool,
    easifemLibs: str,
    buildPath: str,
    installPrefix: str,
    progress,
):

    """
    Parse markdown file and run the app
    """

    job = progress.add_task("[red]" + targetName, total=5)

    outputAbsParent, outputAbsPath, outputFileBase, outputFileExt = _actOnFile(
        inputAbsPath + ".F90",
        check=False,
    )
    progress.advance(job)

    cargs = ["md2src", "-i", inputAbsPath.strip() + ".md"]

    if not quite:
        console.print(f"\n=> Following source file will be generated:")
        console.print(f"=> Absolute path: ", outputAbsPath + outputFileExt)
        console.print(" Calling md2src with following args ...")
        console.print("=> " + "".join(cargs))

    runCommand(cargs, quite=quite, id="(Parsing  )")

    progress.advance(job)

    _actOnF90File(
        outputAbsParent,
        outputAbsPath,
        outputFileBase,
        installPrefix,
        buildPath,
        targetName,
        projectName,
        easifemLibs,
        quite,
        progress,
        job,
    )


def _actOnFortranFile(
    inputAbsPath: str,
    projectName: str,
    targetName: str,
    quite: bool,
    easifemLibs: str,
    buildPath: str,
    installPrefix: str,
    progress,
):
    job = progress.add_task("[red]" + targetName, total=5)

    outputAbsParent, outputAbsPath, outputFileBase, outputFileExt = _actOnFile(
        inputAbsPath + ".F90",
        check=False,
    )
    progress.advance(job)

    _actOnF90File(
        outputAbsParent,
        outputAbsPath,
        outputFileBase,
        installPrefix,
        buildPath,
        targetName,
        projectName,
        easifemLibs,
        quite,
        progress,
        job,
    )



#!/usr/bin/env python3

import argparse
from easifem.utils import easifem_run as run
from easifem.utils_setenv import easifem_setenv as setenv
from easifem.utils_install import easifem_install as install
from easifem.parsers import *
import os

parser = argparse.ArgumentParser(
    prog="python3 easifem.py",
    description="""
    easifem.py is a CLI (Command Line Interface) to libeasifem.
    It contains some subcommands to help you work with libeasifem.
    It can help you in building an application based on easifem components.
    It can also build and install the easifem library.
    For more information visit:
    website: www.easifem.com: 
    (c) Vikas Sharma, 2023
    """,
    epilog="www.easifem.com, (c) Vikas Sharma, 2023",
)

subparser = parser.add_subparsers(
    title="subcommand",
    dest="subcommand",
    prog="easifem.py test [options]",
    required=False,
)


def easifem_test(args):
    run(
        file=args.file,
        quite=args.quite,
        targetName=args.target if args.target else "test",
        easifemLibs=args.easifem_libs if args.easifem_libs else "easifemBase",
        installPrefix=args.prefix if args.prefix else "*--testing--*",
    )


def easifem_run(args):
    run(
        file=args.file,
        quite=args.quite,
        targetName=args.target if args.target else "test",
        easifemLibs=args.easifem_libs if args.easifem_libs else "easifemBase",
        installPrefix=args.prefix if args.prefix else ".",
    )


def easifem_setenv(args):
    setenv(
        quite=args.quite,
        install=args.install if args.install else os.environ["HOME"],
        build=args.build if args.build else os.path.join(os.environ["HOME"], "temp"),
        source=args.source if args.source else os.path.join(os.environ["HOME"], "code"),
        shell=args.shell if args.shell else "bash",
    )


def easifem_install(args):
    install(
        components=args.components,
        quite=args.quite,
        install=args.install if args.install else None,
        build=args.build if args.build else None,
        source=args.source if args.source else None,
    )


parser_test = testParser(subparser)
parser_run = runParser(subparser)
parser_setenv = setenvParser(subparser)
parser_install = installParser(subparser)


def main():
    args = parser.parse_args()
    if args.subcommand:
        if args.subcommand == "test":
            easifem_test(args)
        elif args.subcommand == "run":
            easifem_run(args)
        elif args.subcommand == "setenv":
            easifem_setenv(args)
        elif args.subcommand == "install":
            easifem_install(args)


if __name__ == "__main__":
    main()



version = "2023.1.0"
year = "2020-2023"
lic = (
    f"# This program is a part of",
    f"# EASIFEM",
    f"# version {version}",
    f"# Copyright (C) {year} Vikas Sharma",
    "# Website: www.easifem.com",
    "#",
    "# This program is free software: you can redistribute it and/or modify",
    "# it under the terms of the GNU General Public License as published by",
    "# the Free Software Foundation, either version 3 of the License, or",
    "# (at your option) any later version.",
    "#",
    "# This program is distributed in the hope that it will be useful,",
    "# but WITHOUT ANY WARRANTY; without even the implied warranty of",
    "# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the",
    "# GNU General Public License for more details.",
    "#",
    "# You should have received a copy of the GNU General Public License",
    "# along with this program.  If not, see <https: //www.gnu.org/licenses/>",
    "#",
)

cmake_version = "3.20.0"

_gfortranArgs = (
    '"-ffree-form"',
    '"-ffree-line-length-none"',
    '"-std=f2008"',
    '"-fimplicit-none"',
)

_gfortranArgsDebug = (
    '"-fbounds-check"',
    '"-g" "-fbacktrace" "-Wextra" "-Wall" "-fprofile-arcs"',
    '"-ftest-coverage" "-Wimplicit-interface"',
)

_iforArgs = ('"-r8" "-W1"',)
_iforArgsDebug = (
    '"-O0" "-traceback" "-g" "-debug all"',
    '"-check all" "-ftrapuv" "-warn" "nointerfaces"',
)

_cmakePrefixPath = (
    '"$ENV{EASIFEM_EXTPKGS}"',
    '"$ENV{EASIFEM_BASE}"',
    '"$ENV{EASIFEM_CLASSES}"',
    '"$ENV{EASIFEM_MATERIALS}"',
    '"$ENV{EASIFEM_KERNELS}"',
)

_extLibs = (
    "LAPACK95",
    "Sparsekit",
    "arpackng",
    "OpenMP",
)

_easifemLibs = ("easifemBase",)


def cmakeListBase(
    projectName: str = "easifemApp",
    srcPath: str = "./",
    targetName: str = "test",
    easifemLibs: str = _easifemLibs,
    extLibs: str = _extLibs,
    cmakePrefixPath: str = _cmakePrefixPath,
):
    return (
        *lic,
        f"CMAKE_MINIMUM_REQUIRED(VERSION {cmake_version} FATAL_ERROR)",
        f'SET(PROJECT_NAME "{ projectName }")',
        f'SET(PROJECT_SRC_PATH "{srcPath}")',
        "PROJECT(${PROJECT_NAME})",
        "ENABLE_LANGUAGE(Fortran)",
        f'SET(TARGET_NAME "{targetName}")',
        "",
        "IF(NOT CMAKE_BUILD_TYPE)",
        '  SET(CMAKE_BUILD_TYPE Debug CACHE STRING "Build type" FORCE)',
        "ENDIF()",
        "",
        "IF (",
        '${CMAKE_Fortran_COMPILER_ID} STREQUAL "GNU"',
        'OR Fortran_COMPILER_NAME MATCHES "gfortran*")',
        "LIST(",
        "APPEND",
        "FORTRAN_FLAGS",
        *_gfortranArgs,
        ")",
        'LIST(APPEND FORTRAN_FLAGS_RELEASE "-O3" )',
        "LIST(",
        "APPEND",
        "FORTRAN_FLAGS_DEBUG",
        *_gfortranArgsDebug,
        ")",
        "",
        "ELSEIF(",
        '${CMAKE_Fortran_COMPILER_ID} STREQUAL "Intel"',
        'OR Fortran_COMPILER_NAME MATCHES "ifort*")',
        "LIST(APPEND FORTRAN_FLAGS",
        *_iforArgs,
        ")",
        'LIST(APPEND FORTRAN_FLAGS_RELEASE "-O3")',
        "LIST(APPEND FORTRAN_FLAGS_DEBUG",
        *_iforArgsDebug,
        ")",
        "",
        "ELSEIF (",
        '${CMAKE_Fortran_COMPILER_ID} STREQUAL "XL"',
        'OR Fortran_COMPILER_NAME MATCHES "xlf*")',
        "LIST(APPEND FORTRAN_FLAGS",
        '"-q64" "-qrealsize=8"',
        '"-qsuffix=f=f90:cpp=f90")',
        "LIST(APPEND FORTRAN_FLAGS_RELEASE",
        '"-O3" "-qstrict")',
        "LIST(APPEND FORTRAN_FLAGS_DEBUG",
        '"-O0" "-g" "-qfullpath" "-qkeepparm")',
        "",
        "ELSE ()",
        'MESSAGE(ERROR "No optimized Fortran compiler flags are known")',
        "ENDIF ()",
        "",
        "LIST(",
        "APPEND",
        "CMAKE_PREFIX_PATH",
        *cmakePrefixPath,
        ")",
        "",
        "LIST(",
        "  APPEND",
        "  EXT_LIBS",
        *extLibs,
        ")",
        "",
        "FOREACH(LIB ${EXT_LIBS})",
        "  FIND_PACKAGE(${LIB} REQUIRED)",
        "ENDFOREACH()",
        "",
        "LIST( ",
        "  APPEND ",
        "  EASIFEM_LIBS ",
        *easifemLibs,
        ") ",
        "",
        "FOREACH(LIB ${EASIFEM_LIBS})",
        "  FIND_PACKAGE(${LIB})",
        "ENDFOREACH()",
        "",
        'OPTION( TESTID, "Test number" )',
        'OPTION( CLASSID, "Testing component" )',
        "",
        "IF(NOT CLASSID)",
        "  ADD_EXECUTABLE(${TARGET_NAME} ${FILE})",
        "ELSE( )",
        "  ADD_EXECUTABLE(${TARGET_NAME}",
        "    ${PROJECT_SRC_PATH}/${CLASSID}${PROJECT_NAME}${TESTID}.F90)",
        "ENDIF()",
        "",
        "TARGET_LINK_LIBRARIES( ${TARGET_NAME}",
        f"{easifemLibs[-1]}::{easifemLibs[-1]} )",
        "",
    )



def parser(subparser):

    ans = subparser.add_parser(
        "run",
        help="Run an app build using easifem components, easifemBase, easifemClasses",
        description="""
        The [run] subcommand compiles and links an application, which is build
        by using easifem components, such as, easifemBase, easifemClasses, etc.
        This program can also PARSE a fortran code kept in the code-fences of 
        markdown file with extension [.md]
        The example of code fence which can be used in the markdown file is given below.

        ---- markdown file hello.md ---

        ```fortran
            program main
            use easifemBase
        ```

        Any thing outside fortran code fence will be ignored by the parser.

        Also you can use as many fences as you want, good for Documentation.

        ```fortran
            call display("Hello world")
            end program main
        ```

        To run this file  you can use:

        python3 easifem.py -f hello.md
        """,
    )

    ans.add_argument(
        "-f",
        "--file",
        help="Input filename, e.g. foo.md, ./foo.md, /path/to/foo.md",
    )

    ans.add_argument(
        "-q",
        "--quite",
        help="If specified lesser output will be printed",
        action="store_true",
    )

    ans.add_argument(
        "-t",
        "--target",
        help="Target name, optional[test]",
        required=False,
    )

    ans.add_argument(
        "-e",
        "--easifem_libs",
        help="EASIFEM application library used to build, choices, [easifemBase, easifemClasses, easifemMaterials, easifemKernels, StokesFlow]",
        required=False,
        choices=[
            "easifemBase",
            "easifemClasses",
            "easifemMaterials",
            "easifemKernels",
            "StokesFlow",
            "MovingMesh",
        ],
    )

    ans.add_argument(
        "--prefix",
        help="""
        Install location of source file generated from the markdown parser (md2src). The program will be run from this location. Default is parent directory of markdown file. 
        """,
        required=False,
    )

    return ans



from rich.console import Console

console = Console(soft_wrap=True, record=True)
errConsole = Console(stderr=True, style="bold red", soft_wrap=True)
print = console.print
log = console.log
status = console.log
